@@ -160,31 +160,18 @@ editor_application_open (GApplication  *application,
   g_assert (EDITOR_IS_APPLICATION (self));
   g_assert (files != NULL || n_files == 0);
 
-  /* If we're being asked to open files via this interface,
-   * we want to ignore restoring the previous session because
-   * the user has either provided a filename on the command line
-   * or has opened a file from something like Nautilus or another
-   * default file-handler situation.
-   *
-   * We either have 1) already restored a session, or 2) dont
-   * need to and instead should just show the window. However,
-   * in the case of #2, we need to at least restore our recent
-   * files so the sidebar can continue working. This is done by
-   * setting the restore_pages within EditorSession to FALSE.
-   */
-
   if (_editor_session_did_restore (self->session))
     {
       editor_session_open_files (self->session, files, n_files, hint);
-      return;
     }
-
-  g_application_hold (application);
-  _editor_session_set_restore_pages (self->session, FALSE);
-  editor_session_restore_async (self->session,
-                                NULL,
-                                editor_application_restore_cb,
-                                restore_new (files, n_files, hint));
+  else
+    {
+      g_application_hold (application);
+      editor_session_restore_async (self->session,
+                                    NULL,
+                                    editor_application_restore_cb,
+                                    restore_new (files, n_files, hint));
+    }
 }
 
 static gboolean
@@ -344,6 +331,9 @@ editor_application_startup (GApplication *application)
                                               GTK_STYLE_PROVIDER (self->recoloring),
                                               GTK_STYLE_PROVIDER_PRIORITY_THEME+1);
 
+  gtk_icon_theme_add_resource_path (gtk_icon_theme_get_for_display (display),
+                                    "/org/gnome/TextEditor/icons");
+
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.quit", quit_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.help", help_accels);
 
@@ -405,6 +395,25 @@ editor_application_startup (GApplication *application)
   gtk_window_set_default_icon_name (PACKAGE_ICON_NAME);
 }
 
+static gboolean
+load_stdin_stream_cb (gpointer user_data)
+{
+  EditorApplication *self = EDITOR_APPLICATION_DEFAULT;
+  EditorSession *session;
+  EditorWindow *window;
+  GInputStream *stream = user_data;
+
+  g_assert (EDITOR_IS_APPLICATION (self));
+  g_assert (G_IS_INPUT_STREAM (stream));
+
+  window = editor_application_get_current_window (self);
+  session = editor_application_get_session (self);
+  editor_session_open_stream (session, window, stream);
+  g_application_release (G_APPLICATION (self));
+
+  return G_SOURCE_REMOVE;
+}
+
 static int
 editor_application_command_line (GApplication            *app,
                                  GApplicationCommandLine *command_line)
@@ -412,6 +421,7 @@ editor_application_command_line (GApplication            *app,
   EditorApplication *self = (EditorApplication *)app;
   g_auto(GStrv) argv = NULL;
   g_autoptr(GPtrArray) files = NULL;
+  g_autoptr(GInputStream) stdin_stream = NULL;
   GVariantDict *options;
   gboolean new_window = FALSE;
   const char *hint = NULL;
@@ -430,8 +440,28 @@ editor_application_command_line (GApplication            *app,
   files = g_ptr_array_new_with_free_func (g_object_unref);
 
   for (int i = 1; i < argc; i++)
-    g_ptr_array_add (files,
-                     g_application_command_line_create_file_for_arg (command_line, argv[i]));
+    {
+      /* We want to read stdin into temporary file if we get '-' */
+      if (g_strcmp0 (argv[i], "-") == 0)
+        {
+          if (stdin_stream != NULL)
+            g_application_command_line_printerr (command_line,
+                                                 "%s\n",
+                                                 _("Standard input was requested multiple times. Ignoring request."));
+          else if (!(stdin_stream = g_application_command_line_get_stdin (command_line)))
+            g_application_command_line_printerr (command_line,
+                                                 "%s\n",
+                                                 _("Standard input is not supported on this platform. Ignoring request."));
+          continue;
+        }
+
+      /* Otherwise add the file to the list of files we need to open, taking
+       * into account the other directory a remote instance could be running
+       * from.
+       */
+      g_ptr_array_add (files,
+                       g_application_command_line_create_file_for_arg (command_line, argv[i]));
+    }
 
   /* Only accept --new-window if this is a remote instance, we already
    * create a window if we're in the same process.
@@ -446,9 +476,31 @@ editor_application_command_line (GApplication            *app,
                         (GFile **)(gpointer)files->pdata,
                         files->len,
                         hint);
+  else if (g_strcmp0 (hint, "new-window") == 0)
+    editor_session_create_window (self->session);
   else
     g_application_activate (app);
 
+  /* We've activated but there is a strong chance that our state has not yet
+   * been restored because that requires reading from disk. Give a bit of a
+   * delay before we process the intput stream for our initial windows to be
+   * created and state restored.
+   *
+   * This is basically a hack, but to do anything else would require more state
+   * tracking in the session manager which is particularly difficult as the
+   * stdin could be coming from another process which has been passed to us
+   * over D-Bus.
+   */
+  if (stdin_stream != NULL)
+    {
+      g_application_hold (G_APPLICATION (self));
+      g_timeout_add_full (G_PRIORITY_DEFAULT,
+                          500 /* msec */,
+                          load_stdin_stream_cb,
+                          g_steal_pointer (&stdin_stream),
+                          g_object_unref);
+    }
+
   return EXIT_SUCCESS;
 }
 
@@ -463,9 +515,12 @@ editor_application_handle_local_options (GApplication *app,
   g_assert (options != NULL);
 
   /* This should only happen in the application launching, not the remote
-   * instance as we don't want to affect something already running.
+   * instance as we don't want to affect something already running. Running
+   * standalone implies --ignore-session so that we don't stomp on any
+   * other session data.
    */
-  if (g_variant_dict_lookup (options, "ignore-session", "b", &ignore_session))
+  if (self->standalone ||
+      g_variant_dict_lookup (options, "ignore-session", "b", &ignore_session))
     _editor_session_set_restore_pages (self->session, FALSE);
 
   return G_APPLICATION_CLASS (editor_application_parent_class)->handle_local_options (app, options);
@@ -475,6 +530,7 @@ static void
 editor_application_constructed (GObject *object)
 {
   EditorApplication *self = (EditorApplication *)object;
+  g_autofree char *description = NULL;
 
   g_assert (EDITOR_IS_APPLICATION (self));
 
@@ -482,8 +538,10 @@ editor_application_constructed (GObject *object)
 
   g_application_set_application_id (G_APPLICATION (self), APP_ID);
   g_application_set_resource_base_path (G_APPLICATION (self), "/org/gnome/TextEditor");
-  g_application_set_flags (G_APPLICATION (self),
-                           G_APPLICATION_HANDLES_OPEN | G_APPLICATION_HANDLES_COMMAND_LINE);
+
+  description = g_strdup_printf ("%s %s", _("Bugs may be reported at:"), PACKAGE_BUGREPORTS);
+  g_application_set_option_context_description (G_APPLICATION (self), description);
+  g_application_set_option_context_parameter_string (G_APPLICATION (self), _("[FILES…]"));
 }
 
 static void
@@ -598,6 +656,8 @@ editor_application_class_init (EditorApplicationClass *klass)
 static const GOptionEntry entries[] = {
   { "ignore-session", 'i', 0, G_OPTION_ARG_NONE, NULL, N_("Do not restore session at startup") },
   { "new-window", 'n', 0, G_OPTION_ARG_NONE, NULL, N_("Open provided files in a new window") },
+  { "standalone", 's', 0, G_OPTION_ARG_NONE, NULL, N_("Run a new instance of Text Editor (implies --ignore-session)") },
+  { "version", 0, 0, G_OPTION_ARG_NONE, NULL, N_("Print version information and exit") },
   { 0 }
 };
 
@@ -622,9 +682,20 @@ editor_application_init (EditorApplication *self)
 }
 
 EditorApplication *
-_editor_application_new (void)
+_editor_application_new (gboolean standalone)
 {
-  return g_object_new (EDITOR_TYPE_APPLICATION, NULL);
+  GApplicationFlags flags = G_APPLICATION_HANDLES_COMMAND_LINE | G_APPLICATION_HANDLES_OPEN;
+  EditorApplication *self;
+
+  if (standalone)
+    flags |= G_APPLICATION_NON_UNIQUE;
+
+  self = g_object_new (EDITOR_TYPE_APPLICATION,
+                       "flags", flags,
+                       NULL);
+  self->standalone = !!standalone;
+
+  return self;
 }
 
 /**
