@@ -22,9 +22,10 @@
 
 #include "config.h"
 
-#include <glib/gi18n.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
+
 #include "editor-application.h"
 #include "editor-buffer-monitor-private.h"
 #include "editor-document-private.h"
@@ -38,6 +39,7 @@
 #define METADATA_SYNTAX     "metadata::gte-syntax"
 #define TITLE_LAST_WORD_POS 20
 #define TITLE_MAX_LEN       100
+#define IS_LARGE_FILE(size) ((size) >= 1024UL*1024UL) /* 1MB */
 
 struct _EditorDocument
 {
@@ -82,7 +84,9 @@ typedef struct
   gint64           modified_at;
   guint            n_active;
 
+  guint            highlight_matching_brackets : 1;
   guint            highlight_syntax : 1;
+  guint            check_spelling : 1;
   guint            has_draft : 1;
   guint            has_file : 1;
 } Load;
@@ -96,6 +100,7 @@ enum {
   PROP_EXTERNALLY_MODIFIED,
   PROP_FILE,
   PROP_HAD_ERROR,
+  PROP_LOADING,
   PROP_SPELL_CHECKER,
   PROP_SUGGEST_ADMIN,
   PROP_TITLE,
@@ -282,18 +287,22 @@ editor_document_load_notify_completed_cb (EditorDocument *self,
                                           GParamSpec     *pspec,
                                           GTask          *task)
 {
-  EditorSession *session;
-
   g_assert (EDITOR_IS_DOCUMENT (self));
   g_assert (G_IS_TASK (task));
 
   self->loading = FALSE;
 
+  editor_text_buffer_spell_adapter_invalidate_all (self->spell_adapter);
+
   if (!g_task_had_error (task))
     editor_document_track_error (self, NULL);
 
-  session = editor_application_get_session (EDITOR_APPLICATION_DEFAULT);
-  _editor_session_document_seen (session, self);
+  _editor_session_document_seen (EDITOR_SESSION_DEFAULT, self);
+
+  /* Notify position so that consumers update */
+  g_signal_emit_by_name (self, "cursor-moved");
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_LOADING]);
 }
 
 static void
@@ -326,8 +335,11 @@ editor_document_changed (GtkTextBuffer *buffer)
 
   g_assert (EDITOR_IS_DOCUMENT (self));
 
-  /* Track separately from :modified for drafts */
-  self->needs_autosave = TRUE;
+  if (!self->loading)
+    {
+      /* Track separately from :modified for drafts */
+      self->needs_autosave = TRUE;
+    }
 
   GTK_TEXT_BUFFER_CLASS (editor_document_parent_class)->changed (buffer);
 }
@@ -381,6 +393,15 @@ editor_document_insert_text (GtkTextBuffer *buffer,
   g_assert (pos != NULL);
   g_assert (new_text != NULL);
 
+  /* Short-circuit during loading, we'll handle the followup
+   * actions after loading has completed.
+   */
+  if (self->loading)
+    {
+      GTK_TEXT_BUFFER_CLASS (editor_document_parent_class)->insert_text (buffer, pos, new_text, new_text_length);
+      return;
+    }
+
   line = gtk_text_iter_get_line (pos);
   offset = gtk_text_iter_get_offset (pos);
   length = g_utf8_strlen (new_text, new_text_length);
@@ -414,6 +435,16 @@ editor_document_delete_range (GtkTextBuffer *buffer,
   g_assert (end != NULL);
   g_assert (gtk_text_iter_compare (start, end) <= 0);
 
+  /* Unlikely, but short circuit during loading incase something
+   * beneath us has to delete/replace as part of resolving invalid
+   * characters.
+   */
+  if (self->loading)
+    {
+      GTK_TEXT_BUFFER_CLASS (editor_document_parent_class)->delete_range (buffer, start, end);
+      return;
+    }
+
   offset = gtk_text_iter_get_offset (start);
   length = gtk_text_iter_get_offset (end) - offset;
 
@@ -503,6 +534,9 @@ on_cursor_moved_cb (EditorDocument *self)
   GtkTextMark *mark;
   GtkTextIter iter;
 
+  if (_editor_document_get_loading (self))
+    return;
+
   mark = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (self));
   gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (self), &iter, mark);
 
@@ -596,6 +630,10 @@ editor_document_get_property (GObject    *object,
       g_value_set_object (value, editor_document_get_file (self));
       break;
 
+    case PROP_LOADING:
+      g_value_set_boolean (value, _editor_document_get_loading (self));
+      break;
+
     case PROP_TITLE:
       g_value_take_string (value, editor_document_dup_title (self));
       break;
@@ -689,6 +727,13 @@ editor_document_class_init (EditorDocumentClass *klass)
                          G_TYPE_FILE,
                          (G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
+  properties [PROP_LOADING] =
+    g_param_spec_boolean ("loading",
+                          "Loading",
+                          "If the document is currently loading",
+                          FALSE,
+                          (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
   properties [PROP_SPELL_CHECKER] =
     g_param_spec_object ("spell-checker",
                          "Spell Checker",
@@ -1243,6 +1288,14 @@ _editor_document_unmark_busy (EditorDocument *self)
     }
 }
 
+gboolean
+_editor_document_get_loading (EditorDocument *self)
+{
+  g_return_val_if_fail (EDITOR_IS_DOCUMENT (self), FALSE);
+
+  return self->loading;
+}
+
 gboolean
 editor_document_get_busy (EditorDocument *self)
 {
@@ -1279,6 +1332,7 @@ editor_document_query_info_cb (GObject      *object,
   const gchar *syntax;
   gboolean readonly;
   gboolean is_modified;
+  goffset size;
   guint line = 0;
   guint line_offset = 0;
   Load *load;
@@ -1306,6 +1360,7 @@ editor_document_query_info_cb (GObject      *object,
   position = g_file_info_get_attribute_string (info, METADATA_CURSOR);
   spelling_language = g_file_info_get_attribute_string (info, METADATA_SPELLING);
   syntax = g_file_info_get_attribute_string (info, METADATA_SYNTAX);
+  size = g_file_info_get_size (info);
 
   editor_document_set_readonly (self, readonly);
 
@@ -1355,9 +1410,31 @@ editor_document_query_info_cb (GObject      *object,
 
   editor_buffer_monitor_reset (self->monitor);
 
-  gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (self), load->highlight_syntax);
-  editor_text_buffer_spell_adapter_set_enabled (self->spell_adapter,
-                                                g_settings_get_boolean (shared_settings, "spellcheck"));
+  /* Check to see if we should disable features that are likely to
+   * cause problems on large buffers. We'll consider anything >= 1MB
+   * a "large file" from this standpoint.
+   */
+  if (IS_LARGE_FILE (size))
+    {
+      g_autofree char *title = editor_document_dup_title (self);
+      g_autofree char *sizestr = g_format_size (size);
+
+      g_debug ("Disabling features for buffer \"%s\" due to size (%s)",
+               title, sizestr);
+
+      /* Document settings may try to override these as the load completes
+       * so we need to clear them again.
+       */
+      gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (self), FALSE);
+      gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (self), FALSE);
+      editor_text_buffer_spell_adapter_set_enabled (self->spell_adapter, FALSE);
+    }
+  else
+    {
+      gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (self), load->highlight_matching_brackets);
+      gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (self), load->highlight_syntax);
+      editor_text_buffer_spell_adapter_set_enabled (self->spell_adapter, load->check_spelling);
+    }
 
   _editor_document_unmark_busy (self);
 
@@ -1408,6 +1485,7 @@ editor_document_load_cb (GObject      *object,
         file = draft_file = editor_document_get_draft_file (self);
 
       g_file_query_info_async (file,
+                               G_FILE_ATTRIBUTE_STANDARD_SIZE","
                                G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE","
                                G_FILE_ATTRIBUTE_FILESYSTEM_READONLY","
                                METADATA_CURSOR","
@@ -1437,9 +1515,9 @@ editor_document_do_load (EditorDocument *self,
       /* We are creating a new file. */
       editor_document_set_busy_progress (self, 1, 2, 1.0);
       _editor_document_unmark_busy (self);
-      gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (self), TRUE);
-      editor_text_buffer_spell_adapter_set_enabled (self->spell_adapter,
-                                                    g_settings_get_boolean (shared_settings, "spellcheck"));
+      gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (self), load->highlight_matching_brackets);
+      gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (self), load->highlight_syntax);
+      editor_text_buffer_spell_adapter_set_enabled (self->spell_adapter, load->check_spelling);
       g_task_return_boolean (task, TRUE);
       return;
     }
@@ -1656,6 +1734,7 @@ _editor_document_load_async (EditorDocument      *self,
   g_return_if_fail (self->loading == FALSE);
 
   self->loading = TRUE;
+  self->needs_autosave = FALSE;
 
   file = editor_document_get_file (self);
 
@@ -1668,8 +1747,12 @@ _editor_document_load_async (EditorDocument      *self,
   else
     load->mount_operation = g_mount_operation_new ();
 
-  if (gtk_source_buffer_get_highlight_syntax (GTK_SOURCE_BUFFER (self)))
-    load->highlight_syntax = TRUE;
+  load->highlight_matching_brackets =
+      gtk_source_buffer_get_highlight_matching_brackets (GTK_SOURCE_BUFFER (self));
+  load->highlight_syntax =
+      gtk_source_buffer_get_highlight_syntax (GTK_SOURCE_BUFFER (self));
+  load->check_spelling =
+      editor_text_buffer_spell_adapter_get_enabled (self->spell_adapter);
 
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, _editor_document_load_async);
@@ -1689,6 +1772,7 @@ _editor_document_load_async (EditorDocument      *self,
    * and reduce the chances that syntax highlights can hit scenarios
    * where text changes during the main loop idle callbacks.
    */
+  gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (self), FALSE);
   gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (self), FALSE);
   editor_text_buffer_spell_adapter_set_enabled (self->spell_adapter, FALSE);
 
@@ -1717,6 +1801,8 @@ _editor_document_load_async (EditorDocument      *self,
                                      editor_document_load_file_mount_cb,
                                      g_object_ref (task));
     }
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_LOADING]);
 }
 
 gboolean
@@ -1741,9 +1827,10 @@ editor_document_guess_language_query_cb (GObject      *object,
   g_autoptr(GTask) task = user_data;
   g_autofree gchar *filename = NULL;
   GtkSourceLanguageManager *lm;
-  GtkSourceLanguage *language;
+  GtkSourceLanguage *language = NULL;
   EditorDocument *self;
   const gchar *content_type;
+  const char *syntax;
 
   g_assert (G_IS_FILE (file));
   g_assert (G_IS_ASYNC_RESULT (result));
@@ -1755,14 +1842,21 @@ editor_document_guess_language_query_cb (GObject      *object,
       return;
     }
 
-  content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
-  filename = g_file_get_basename (file);
   lm = gtk_source_language_manager_get_default ();
-  language = guess_language (lm, filename, content_type);
+
+  if ((syntax = g_file_info_get_attribute_string (info, METADATA_SYNTAX)))
+    language = gtk_source_language_manager_get_language (lm, syntax);
+
+  if (language == NULL)
+    {
+      content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+      filename = g_file_get_basename (file);
+      language = guess_language (lm, filename, content_type);
+    }
+
   self = g_task_get_source_object (task);
 
   g_assert (EDITOR_IS_DOCUMENT (self));
-  g_assert (GTK_SOURCE_IS_LANGUAGE_MANAGER (lm));
   g_assert (!language || GTK_SOURCE_IS_LANGUAGE (language));
 
   if (language != NULL)
@@ -1808,7 +1902,8 @@ _editor_document_guess_language_async (EditorDocument      *self,
                              "Cannot query file as load failed.");
   else
     g_file_query_info_async (file,
-                             G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+                             G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE","
+                             METADATA_SYNTAX,
                              G_FILE_QUERY_INFO_NONE,
                              G_PRIORITY_DEFAULT,
                              cancellable,
@@ -2165,7 +2260,8 @@ _editor_document_get_spelling_tag (EditorDocument *self)
 }
 
 void
-_editor_document_use_admin (EditorDocument *self)
+_editor_document_use_admin (EditorDocument *self,
+                            EditorWindow   *window)
 {
   GFile *file;
   g_autofree char *uri = NULL;
@@ -2175,18 +2271,27 @@ _editor_document_use_admin (EditorDocument *self)
   g_return_if_fail (EDITOR_IS_DOCUMENT (self));
 
   if (!(file = editor_document_get_file (self)))
-    return;
+    {
+      g_warning ("No file, cannot change to admin:// protocol");
+      return;
+    }
 
   uri = g_file_get_uri (file);
+
   if (!g_str_has_prefix (uri, "file:///"))
-    return;
+    {
+      g_warning ("URI \"%s\" does not start with \"file:///\"", uri);
+      return;
+    }
 
   admin_uri = g_strdup_printf ("admin://%s", g_file_get_path (file));
   admin_file = g_file_new_for_uri (admin_uri);
 
+  g_debug ("Changing URI from \"%s\" to \"%s\"", uri, admin_uri);
   gtk_source_file_set_location (self->file, admin_file);
-
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_FILE]);
+
+  _editor_document_load_async (self, window, NULL, NULL, NULL);
 }
 
 gboolean
@@ -2196,3 +2301,64 @@ _editor_document_had_error (EditorDocument *self)
 
   return self->load_failed;
 }
+
+char *
+_editor_document_suggest_filename (EditorDocument *self)
+{
+  g_autofree char *title = NULL;
+  GtkSourceLanguage *lang;
+  const char *suggested_name = NULL;
+  const char *suggested_suffix = NULL;
+  const char *current_suffix;
+
+  g_assert (EDITOR_IS_DOCUMENT (self));
+
+  if ((lang = gtk_source_buffer_get_language (GTK_SOURCE_BUFFER (self))))
+    {
+      suggested_suffix = gtk_source_language_get_metadata (lang, "suggested-suffix");
+      suggested_name = gtk_source_language_get_metadata (lang, "suggested-name");
+    }
+
+  if (suggested_name != NULL)
+    return g_strdup (suggested_name);
+
+  if (suggested_suffix == NULL)
+    suggested_suffix = ".txt";
+
+  title = editor_document_dup_title (self);
+
+  if (title == NULL)
+    title = g_strdup (_("New Document"));
+
+  current_suffix = strrchr (title, '.');
+
+  if (g_strcmp0 (current_suffix, suggested_suffix) == 0)
+    return g_steal_pointer (&title);
+
+  return g_strdup_printf ("%s%s", title, suggested_suffix);
+}
+
+GFile *
+_editor_document_suggest_file (EditorDocument *self,
+                               GFile          *directory)
+{
+  static GFile *documents;
+  g_autofree char *name = NULL;
+
+  g_return_val_if_fail (EDITOR_IS_DOCUMENT (self), NULL);
+  g_return_val_if_fail (!directory || G_IS_FILE (directory), NULL);
+
+  if (directory == NULL)
+    {
+      if (documents == NULL)
+        documents = g_file_new_for_path (g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS));
+      directory = documents;
+    }
+
+  name = _editor_document_suggest_filename (self);
+
+  g_assert (G_IS_FILE (directory));
+  g_assert (name != NULL);
+
+  return g_file_get_child (directory, name);
+}
