@@ -43,6 +43,7 @@ struct _EditorSourceView
   char *spelling_word;
   int font_scale;
   double line_height;
+  guint overscroll_source;
 };
 
 G_DEFINE_TYPE (EditorSourceView, editor_source_view, GTK_SOURCE_TYPE_VIEW)
@@ -58,6 +59,26 @@ enum {
 
 static GParamSpec *properties [N_PROPS];
 
+static gboolean
+editor_source_view_update_overscroll (gpointer user_data)
+{
+  EditorSourceView *self = user_data;
+
+  g_assert (EDITOR_IS_SOURCE_VIEW (self));
+
+  self->overscroll_source = 0;
+
+  if (gtk_widget_get_mapped (GTK_WIDGET (self)))
+    {
+      GdkRectangle visible_rect;
+
+      gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (self), &visible_rect);
+      gtk_text_view_set_bottom_margin (GTK_TEXT_VIEW (self), visible_rect.height * .75);
+    }
+
+  return G_SOURCE_REMOVE;
+}
+
 static void
 editor_source_view_update_css (EditorSourceView *self)
 {
@@ -693,6 +714,56 @@ editor_source_view_snapshot_layer (GtkTextView      *text_view,
     editor_source_view_draw_search_bubbles (self, snapshot);
 }
 
+static gboolean
+editor_source_view_scroll_to_insert_in_idle_cb (gpointer user_data)
+{
+  EditorSourceView *self = user_data;
+  GtkTextBuffer *buffer;
+  GtkTextMark *mark;
+  GtkTextIter iter;
+
+  g_assert (EDITOR_IS_SOURCE_VIEW (self));
+
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (self));
+  mark = gtk_text_buffer_get_insert (buffer);
+  gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark);
+
+  editor_source_view_jump_to_iter (GTK_TEXT_VIEW (self), &iter, .25, TRUE, 1.0, 0.5);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+editor_source_view_root (GtkWidget *widget)
+{
+  EditorSourceView *self = (EditorSourceView *)widget;
+
+  g_assert (EDITOR_IS_SOURCE_VIEW (self));
+
+  GTK_WIDGET_CLASS (editor_source_view_parent_class)->root (widget);
+
+  g_idle_add_full (G_PRIORITY_LOW,
+                   editor_source_view_scroll_to_insert_in_idle_cb,
+                   g_object_ref (self),
+                   g_object_unref);
+}
+
+static void
+editor_source_view_size_allocate (GtkWidget *widget,
+                                  int        width,
+                                  int        height,
+                                  int        baseline)
+{
+  EditorSourceView *self = (EditorSourceView *)widget;
+
+  g_assert (EDITOR_IS_SOURCE_VIEW (self));
+
+  GTK_WIDGET_CLASS (editor_source_view_parent_class)->size_allocate (widget, width, height, baseline);
+
+  if (self->overscroll_source == 0)
+    self->overscroll_source = g_idle_add (editor_source_view_update_overscroll, self);
+}
+
 static void
 editor_source_view_constructed (GObject *object)
 {
@@ -708,6 +779,11 @@ editor_source_view_dispose (GObject *object)
 {
   EditorSourceView *self = (EditorSourceView *)object;
 
+  g_clear_handle_id (&self->overscroll_source, g_source_remove);
+
+  gtk_text_view_set_extra_menu (GTK_TEXT_VIEW (self), NULL);
+  gtk_widget_insert_action_group (GTK_WIDGET (self), "spelling", NULL);
+
   g_clear_object (&self->css_provider);
   g_clear_object (&self->spelling_menu);
   g_clear_pointer (&self->spelling_word, g_free);
@@ -788,6 +864,9 @@ editor_source_view_class_init (EditorSourceViewClass *klass)
   object_class->get_property = editor_source_view_get_property;
   object_class->set_property = editor_source_view_set_property;
 
+  widget_class->root = editor_source_view_root;
+  widget_class->size_allocate = editor_source_view_size_allocate;
+
   text_view_class->snapshot_layer = editor_source_view_snapshot_layer;
 
   properties [PROP_LINE_HEIGHT] =
@@ -886,7 +965,8 @@ editor_source_view_init (EditorSourceView *self)
                     self);
   gtk_widget_add_controller (GTK_WIDGET (self), controller);
 
-  controller = gtk_event_controller_scroll_new (GTK_EVENT_CONTROLLER_SCROLL_VERTICAL | GTK_EVENT_CONTROLLER_SCROLL_DISCRETE);
+  controller = gtk_event_controller_scroll_new (GTK_EVENT_CONTROLLER_SCROLL_VERTICAL);
+  gtk_event_controller_set_propagation_phase (controller, GTK_PHASE_CAPTURE);
   g_signal_connect (controller,
                     "scroll",
                     G_CALLBACK (on_scroll_scrolled_cb),
@@ -973,3 +1053,143 @@ editor_source_view_prepend_extra_menu (EditorSourceView *self,
 
   editor_joined_menu_prepend_menu (EDITOR_JOINED_MENU (base), extra_menu);
 }
+
+/**
+ * editor_source_view_jump_to_iter:
+ *
+ * The goal of this function is to be like gtk_text_view_scroll_to_iter() but
+ * without any of the scrolling animation. We use it to move to a position
+ * when animations would cause additional distractions.
+ */
+void
+editor_source_view_jump_to_iter (GtkTextView       *text_view,
+                                 const GtkTextIter *iter,
+                                 double             within_margin,
+                                 gboolean           use_align,
+                                 double             xalign,
+                                 double             yalign)
+{
+  GtkAdjustment *hadj;
+  GtkAdjustment *vadj;
+  GdkRectangle rect;
+  GdkRectangle screen;
+  int xvalue = 0;
+  int yvalue = 0;
+  int scroll_dest;
+  int screen_bottom;
+  int screen_right;
+  int screen_xoffset;
+  int screen_yoffset;
+  int current_x_scroll;
+  int current_y_scroll;
+  int top_margin;
+
+  /*
+   * Many parts of this function were taken from gtk_text_view_scroll_to_iter ()
+   * https://developer.gnome.org/gtk3/stable/GtkTextView.html#gtk-text-view-scroll-to-iter
+   */
+
+  g_return_if_fail (GTK_IS_TEXT_VIEW (text_view));
+  g_return_if_fail (iter != NULL);
+  g_return_if_fail (within_margin >= 0.0 && within_margin <= 0.5);
+  g_return_if_fail (xalign >= 0.0 && xalign <= 1.0);
+  g_return_if_fail (yalign >= 0.0 && yalign <= 1.0);
+
+  g_object_get (text_view, "top-margin", &top_margin, NULL);
+
+  hadj = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (text_view));
+  vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (text_view));
+
+  gtk_text_view_get_iter_location (text_view, iter, &rect);
+  gtk_text_view_get_visible_rect (text_view, &screen);
+
+  current_x_scroll = screen.x;
+  current_y_scroll = screen.y;
+
+  screen_xoffset = screen.width * within_margin;
+  screen_yoffset = screen.height * within_margin;
+
+  screen.x += screen_xoffset;
+  screen.y += screen_yoffset;
+  screen.width -= screen_xoffset * 2;
+  screen.height -= screen_yoffset * 2;
+
+
+  /* paranoia check */
+  if (screen.width < 1)
+    screen.width = 1;
+  if (screen.height < 1)
+    screen.height = 1;
+
+  /* The -1 here ensures that we leave enough space to draw the cursor
+   * when this function is used for horizontal scrolling.
+   */
+  screen_right = screen.x + screen.width - 1;
+  screen_bottom = screen.y + screen.height;
+
+
+  /* The alignment affects the point in the target character that we
+   * choose to align. If we're doing right/bottom alignment, we align
+   * the right/bottom edge of the character the mark is at; if we're
+   * doing left/top we align the left/top edge of the character; if
+   * we're doing center alignment we align the center of the
+   * character.
+   */
+
+  /* Vertical alignment */
+  scroll_dest = current_y_scroll;
+  if (use_align)
+    {
+      scroll_dest = rect.y + (rect.height * yalign) - (screen.height * yalign);
+
+      /* if scroll_dest < screen.y, we move a negative increment (up),
+       * else a positive increment (down)
+       */
+      yvalue = scroll_dest - screen.y + screen_yoffset;
+    }
+  else
+    {
+      /* move minimum to get onscreen */
+      if (rect.y < screen.y)
+        {
+          scroll_dest = rect.y;
+          yvalue = scroll_dest - screen.y - screen_yoffset;
+        }
+      else if ((rect.y + rect.height) > screen_bottom)
+        {
+          scroll_dest = rect.y + rect.height;
+          yvalue = scroll_dest - screen_bottom + screen_yoffset;
+        }
+    }
+  yvalue += current_y_scroll;
+
+  /* Horizontal alignment */
+  scroll_dest = current_x_scroll;
+  if (use_align)
+    {
+      scroll_dest = rect.x + (rect.width * xalign) - (screen.width * xalign);
+
+      /* if scroll_dest < screen.y, we move a negative increment (left),
+       * else a positive increment (right)
+       */
+      xvalue = scroll_dest - screen.x + screen_xoffset;
+    }
+  else
+    {
+      /* move minimum to get onscreen */
+      if (rect.x < screen.x)
+        {
+          scroll_dest = rect.x;
+          xvalue = scroll_dest - screen.x - screen_xoffset;
+        }
+      else if ((rect.x + rect.width) > screen_right)
+        {
+          scroll_dest = rect.x + rect.width;
+          xvalue = scroll_dest - screen_right + screen_xoffset;
+        }
+    }
+  xvalue += current_x_scroll;
+
+  gtk_adjustment_set_value (hadj, xvalue);
+  gtk_adjustment_set_value (vadj, yvalue + top_margin);
+}
