@@ -27,6 +27,7 @@
 #include "editor-application.h"
 #include "editor-document.h"
 #include "editor-open-popover-private.h"
+#include "editor-page-private.h"
 #include "editor-save-changes-dialog-private.h"
 #include "editor-session-private.h"
 #include "editor-theme-selector-private.h"
@@ -40,7 +41,7 @@ typedef struct
   const GtkSourceEncoding *encoding;
 } ClosedItem;
 
-G_DEFINE_TYPE (EditorWindow, editor_window, ADW_TYPE_APPLICATION_WINDOW)
+G_DEFINE_FINAL_TYPE (EditorWindow, editor_window, ADW_TYPE_APPLICATION_WINDOW)
 
 enum {
   PROP_0,
@@ -587,9 +588,18 @@ editor_window_actions_close_page_confirm_cb (GObject      *object,
   for (guint i = 0; i < pages->len; i++)
     {
       EditorPage *epage = g_ptr_array_index (pages, i);
-      AdwTabPage *page = adw_tab_view_get_page (self->tab_view, GTK_WIDGET (epage));
+      AdwTabPage *page;
 
       g_assert (EDITOR_IS_PAGE (epage));
+
+      /* Make sure the page is still around and attached across the async
+       * operations. Otherwise, skip it as it's already cleaned up.
+       */
+      if (gtk_widget_get_ancestor (GTK_WIDGET (epage), ADW_TYPE_TAB_VIEW) != GTK_WIDGET (self->tab_view))
+        continue;
+
+      page = adw_tab_view_get_page (self->tab_view, GTK_WIDGET (epage));
+
       g_assert (ADW_IS_TAB_PAGE (page));
 
       if (epage->close_requested)
@@ -714,7 +724,7 @@ notify_focus_widget_cb (EditorWindow *self)
       EditorPage *page = editor_window_get_visible_page (self);
 
       if (page != NULL)
-        gtk_root_set_focus (GTK_ROOT (self), GTK_WIDGET (page));
+        gtk_root_set_focus (GTK_ROOT (self), GTK_WIDGET (page->view));
     }
 }
 
